import unittest

import mock


class TestIncludeme(unittest.TestCase):

    def test_includeme(self):
        from pyramid_google_login import includeme

        config = mock.Mock()

        includeme(config)

        config.include.assert_called_once_with('pyramid_mako')
        expected_calls = [
            mock.call('auth_signin', '/auth/signin'),
            mock.call('auth_signin_redirect', '/auth/signin_redirect'),
            mock.call('auth_callback', '/auth/oauth2callback'),
            mock.call('auth_logout', '/auth/logout'),
        ]
        config.add_route.assert_has_calls(expected_calls, any_order=True)
        config.add_static_view.assert_called_once_with(
            'static/pyramid_google_login',
            'pyramid_google_login:static',
            cache_max_age=300)
        config.scan.assert_called_once_with()


class TestHelpers(unittest.TestCase):

    def test_redirect_to_signin(self):
        from pyramid.httpexceptions import HTTPFound
        from pyramid_google_login import redirect_to_signin

        request = mock.Mock()

        with self.assertRaises(HTTPFound) as test_exc:
            redirect_to_signin(request)

        request.route_url.assert_called_once_with('auth_signin', _query={})
        httpfound = test_exc.exception
        self.assertEqual(httpfound.location, request.route_url.return_value)

    def test_redirect_to_signin_url(self):
        from pyramid.httpexceptions import HTTPFound
        from pyramid_google_login import redirect_to_signin

        request = mock.Mock()

        with self.assertRaises(HTTPFound) as test_exc:
            redirect_to_signin(request, url='/test')

        request.route_url.assert_called_once_with('auth_signin',
                                                  _query={'url': '/test'})
        httpfound = test_exc.exception
        self.assertEqual(httpfound.location, request.route_url.return_value)

    def test_redirect_to_signin_headers(self):
        from pyramid.httpexceptions import HTTPFound
        from pyramid_google_login import redirect_to_signin

        request = mock.Mock()

        test_header = ('X-Test', 'Yeap')

        with self.assertRaises(HTTPFound) as test_exc:
            redirect_to_signin(request, headers=[test_header])

        httpfound = test_exc.exception
        self.assertEqual(httpfound.location, request.route_url.return_value)
        self.assertIn(test_header, httpfound.headerlist)



import unittest

import mock


@mock.patch('pyramid_google_login.views.exchange_token_from_code')
@mock.patch('pyramid_google_login.views.get_userinfo_from_token')
@mock.patch('pyramid_google_login.views.get_principal_from_userinfo')
@mock.patch('pyramid_google_login.views.decode_state')
@mock.patch('pyramid_google_login.views.remember')
class TestCallback(unittest.TestCase):

    def setUp(self):
        self.request = mock.Mock()
        self.request.params = {
            'code': 'CODE1234',
            'state': 'STATE1234'
        }
        self.settings = {
            'security.google_login.client_id': 'CLIENTID',
            'security.google_login.client_secret': 'CLIENTSECRET',
        }
        self.request.registry.settings = self.settings

        self.request.route_url.return_value = '/test/url'

    def test_nominal(self, m_rem, m_dec_s, m_get_p, m_get_u, m_exch_t):
        from pyramid_google_login.views import callback
        from pyramid.httpexceptions import HTTPFound

        m_rem.return_value = [('Set-Cookie', 'auth_tkt=plop')]
        m_dec_s.return_value = {'url': ['/next/url']}

        resp = callback(self.request)

        m_exch_t.assert_called_once_with(self.request)
        m_get_u.assert_called_once_with(m_exch_t.return_value)
        m_get_p.assert_called_once_with(self.request, m_get_u.return_value)
        m_rem.assert_called_once_with(self.request,
                                      principal=m_get_p.return_value,
                                      max_age=24 * 3600)

        self.assertIsInstance(resp, HTTPFound)
        self.assertEqual(resp.location, '/next/url')

        expected_header = ('Set-Cookie', 'auth_tkt=plop')
        self.assertIn(expected_header, resp.headerlist)

    def test_no_url(self, m_rem, m_dec_s, m_get_p, m_get_u, m_exch_t):
        from pyramid_google_login.views import callback
        from pyramid.httpexceptions import HTTPFound

        m_rem.return_value = [('Set-Cookie', 'auth_tkt=plop')]
        m_dec_s.return_value = {}

        resp = callback(self.request)

        self.assertIsInstance(resp, HTTPFound)
        self.assertEqual(resp.location, '/')

    def test_oauth2_flow_error(self, m_rem, m_dec_s, m_get_p, m_get_u,
                               m_exch_t):
        from pyramid_google_login.views import callback
        from pyramid_google_login import AuthFailed

        from pyramid.httpexceptions import HTTPFound

        m_exch_t.side_effect = AuthFailed('TESTEXC')

        with self.assertRaises(HTTPFound) as test_exc:
            callback(self.request)

        self.request.route_url.assert_called_once_with(
            'auth_signin',
            _query={'message': 'Google Login failed (TESTEXC)'})

        self.assertEqual(test_exc.exception.location,
                         '/test/url')

    def test_unkown_error(self, m_rem, m_dec_s, m_get_p, m_get_u,
                          m_exch_t):
        from pyramid_google_login.views import callback

        from pyramid.httpexceptions import HTTPFound

        m_exch_t.side_effect = Exception('TESTEXC')

        with self.assertRaises(HTTPFound) as test_exc:
            callback(self.request)

        self.request.route_url.assert_called_once_with(
            'auth_signin',
            _query={'message': 'Google Login failed (unkown)'})

        self.assertEqual(test_exc.exception.location,
                         '/test/url')



import unittest

import mock


class TestState(unittest.TestCase):

    def test_encode_state(self):
        from pyramid_google_login.google_oauth2 import encode_state

        tested = [('param1', 'value1'),
                  ('param2', 'value2'),
                  ('param2', 'value3')]
        resp = encode_state(tested)
        expected = 'param1=value1&param2=value2&param2=value3'
        self.assertEqual(resp, expected)

    def test_decode_state(self):
        from pyramid_google_login.google_oauth2 import decode_state

        tested = 'param1=value1&param2=value2&param2=value3'
        resp = decode_state(tested)
        expected = {'param1': ['value1'], 'param2': ['value2', 'value3']}
        self.assertEqual(resp, expected)


class TestBuildAuthorizeUrl(unittest.TestCase):

    def test_nominal(self):
        from pyramid_google_login.google_oauth2 import build_authorize_url

        request = mock.Mock()
        request.route_url.return_value = 'TESTROUTEURL'
        request.registry.settings = {
            'security.google_login.client_id': 'CLIENTID',
            'security.google_login.hosted_domain': 'example.net',
        }
        state = 'TESTSTATE'

        url = build_authorize_url(request, state)
        expected = ('https://accounts.google.com/o/oauth2/auth?'
                    'state=TESTSTATE&redirect_uri=TESTROUTEURL&'
                    'response_type=code&client_id=CLIENTID&'
                    'scope=email&hd=example.net')

        self.assertEqual(url, expected)

    def test_missing_setting(self):
        from pyramid_google_login.google_oauth2 import build_authorize_url
        from pyramid_google_login import AuthFailed

        request = mock.Mock()
        request.route_url.return_value = 'TESTROUTEURL'
        request.registry.settings = {}
        state = 'TESTSTATE'

        with self.assertRaises(AuthFailed) as test_exc:
            build_authorize_url(request, state)

        auth_failed = test_exc.exception
        self.assertEqual(auth_failed.message, 'Missing settings')


class TestExchangeTokenFromCode(unittest.TestCase):

    def setUp(self):
        self.request = mock.Mock()
        self.request.params = {'code': 'CODE1234'}
        self.settings = {
            'security.google_login.client_id': 'CLIENTID',
            'security.google_login.client_secret': 'CLIENTSECRET',
        }
        self.request.registry.settings = self.settings

    def call_and_assert_raises_with_message(self, message):
        from pyramid_google_login import AuthFailed
        from pyramid_google_login.google_oauth2 import exchange_token_from_code

        with self.assertRaises(AuthFailed) as test_exc:
            exchange_token_from_code(self.request)

        self.assertEqual(test_exc.exception.message, message)

    @mock.patch('pyramid_google_login.google_oauth2.requests')
    def test_nominal(self, m_requests):
        from pyramid_google_login.google_oauth2 import exchange_token_from_code

        token = exchange_token_from_code(self.request)

        m_json_response = m_requests.post.return_value.json.return_value
        m_token = m_json_response.__getitem__.return_value
        self.assertEqual(token, m_token)

    def test_param_error(self):
        self.request.params = {'error': 'TESTERROR'}
        self.call_and_assert_raises_with_message(
            'Error from Google (TESTERROR)')

    def test_param_no_code(self):
        self.request.params = {}
        self.call_and_assert_raises_with_message(
            'No authorization code from Google')

    def test_missing_client_id(self):
        del self.settings['security.google_login.client_id']
        self.call_and_assert_raises_with_message(
            'Missing settings')

    def test_missing_client_secret(self):
        del self.settings['security.google_login.client_secret']
        self.call_and_assert_raises_with_message(
            'Missing settings')

    @mock.patch('pyramid_google_login.google_oauth2.requests')
    def test_token_endpoint_request_error(self, m_requests):
        from requests.exceptions import RequestException

        m_requests.post.side_effect = RequestException('TESTEXC')

        self.call_and_assert_raises_with_message(
            'Failed to get token from Google (TESTEXC)')

    @mock.patch('pyramid_google_login.google_oauth2.requests')
    def test_token_endpoint_unknown_error(self, m_requests):
        m_requests.post.side_effect = Exception('TESTEXC')

        self.call_and_assert_raises_with_message(
            'Failed to get token from Google (unkown error)')

    @mock.patch('pyramid_google_login.google_oauth2.requests')
    def test_invalid_token_endpoint_response(self, m_requests):
        m_requests.post.return_value.json.return_value = {}

        self.call_and_assert_raises_with_message(
            'No access_token in response from Google')


class TestGetUserinfoFromToken(unittest.TestCase):

    def call_and_assert_raises_with_message(self, message):
        from pyramid_google_login import AuthFailed
        from pyramid_google_login.google_oauth2 import get_userinfo_from_token

        with self.assertRaises(AuthFailed) as test_exc:
            get_userinfo_from_token('TESTTOKEN')

        self.assertEqual(test_exc.exception.message, message)

    @mock.patch('pyramid_google_login.google_oauth2.requests')
    def test_nominal(self, m_requests):
        from pyramid_google_login.google_oauth2 import get_userinfo_from_token

        m_json_response = m_requests.get.return_value.json.return_value

        access_token = 'TESTTOKEN'

        userinfo = get_userinfo_from_token(access_token)

        self.assertEqual(userinfo, m_json_response)

    @mock.patch('pyramid_google_login.google_oauth2.requests')
    def test_userinfo_endpoint_unknown_error(self, m_requests):
        m_requests.get.side_effect = Exception('TESTEXC')

        self.call_and_assert_raises_with_message(
            'Failed to get userinfo from Google')


class TestCheckHostedDomainUser(unittest.TestCase):

    def setUp(self):
        self.request = mock.Mock()
        self.request.params = {'code': 'CODE1234'}
        self.settings = {
            'security.google_login.hosted_domain': 'example.net',
        }
        self.request.registry.settings = self.settings

    def test_nominal(self):
        from pyramid_google_login.google_oauth2 import (
            check_hosted_domain_user)

        userinfo = {'hd': 'example.net'}
        check_hosted_domain_user(self.request, userinfo)

    def test_wrong_domain(self):
        from pyramid_google_login import AuthFailed
        from pyramid_google_login.google_oauth2 import (
            check_hosted_domain_user)

        userinfo = {'hd': 'reallynotexample.net'}

        with self.assertRaises(AuthFailed) as test_exc:
            check_hosted_domain_user(self.request, userinfo)

        self.assertEqual(test_exc.exception.message,
                         'You logged in with an unkown domain '
                         '(reallynotexample.net rather than example.net)')

    def test_missing_userinfo_field(self):
        from pyramid_google_login import AuthFailed
        from pyramid_google_login.google_oauth2 import (
            check_hosted_domain_user)

        userinfo = {}

        with self.assertRaises(AuthFailed) as test_exc:
            check_hosted_domain_user(self.request, userinfo)

        self.assertEqual(test_exc.exception.message,
                         'Missing hd field from Google userinfo')


class TestGetPrincipalFromUserinfo(unittest.TestCase):
    def test_nominal(self):
        from pyramid_google_login.google_oauth2 import (
            get_principal_from_userinfo)

        userinfo = {'email': 'TESTEMAIL', 'id': 'TESTID'}

        request = mock.Mock()

        request.registry.settings = {}
        principal = get_principal_from_userinfo(request, userinfo)
        self.assertEqual(principal, 'TESTEMAIL')

        request.registry.settings = {
            'security.google_login.user_id_field': 'email'}
        principal = get_principal_from_userinfo(request, userinfo)
        self.assertEqual(principal, 'TESTEMAIL')

        request.registry.settings = {
            'security.google_login.user_id_field': 'id'}
        principal = get_principal_from_userinfo(request, userinfo)
        self.assertEqual(principal, 'TESTID')

    def test_missing_field(self):
        from pyramid_google_login import AuthFailed
        from pyramid_google_login.google_oauth2 import (
            get_principal_from_userinfo)

        userinfo = {}

        request = mock.Mock()
        request.registry.settings = {}

        with self.assertRaises(AuthFailed) as test_exc:
            get_principal_from_userinfo(request, userinfo)

        self.assertEqual(test_exc.exception.message,
                         'Missing principal field from Google userinfo')



import unittest
from webtest import TestApp


class Test(unittest.TestCase):
    def set_config(self, settings=None):
        if settings is None:
            settings = {
                'security.google_login.client_id': 'CLIENT_ID',
                'security.google_login.client_secret': 'CLIENT_SECRET',
            }

        from pyramid.config import Configurator
        config = Configurator()
        config.add_settings(settings)

        import pyramid_google_login
        config.include(pyramid_google_login)

        self.app = TestApp(config.make_wsgi_app())

    def test_signin(self):
        self.set_config()
        resp = self.app.get('/auth/signin',
                            status=200)
        expected = '''href="http://localhost/auth/signin_redirect"'''
        self.assertIn(expected, resp.body)

    def test_signin_url(self):
        self.set_config()
        resp = self.app.get('/auth/signin?url=TESTURL',
                            status=200)
        expected = ('''href="http://localhost/auth/signin_redirect'''
                    '''?url=TESTURL"''')
        self.assertIn(expected, resp.body)

    def test_signin_message(self):
        self.set_config()
        resp = self.app.get('/auth/signin?message=TEST+MESSAGE',
                            status=200)
        expected = '''TEST MESSAGE'''
        self.assertIn(expected, resp.body)

    def test_signin_redirect(self):
        self.set_config()
        resp = self.app.get('/auth/signin_redirect?url=TEST%2FURL',
                            status=302)

        expected = (
            'https://accounts.google.com/o/oauth2/auth?'
            'scope=email&state=url%3DTEST%252FURL&'
            'redirect_uri=http%3A%2F%2Flocalhost%2Fauth%2Foauth2callback&'
            'response_type=code&client_id=CLIENT_ID')
        self.assertEqual(resp.location, expected)

    def test_signin_redirect_missing_clientid(self):
        self.set_config({})

        resp = self.app.get('/auth/signin_redirect',
                            status=302)

        expected = ('http://localhost/auth/signin?message='
                    'Google+Login+failed+%28Missing+settings%29')
        self.assertEqual(resp.location, expected)

    def test_callback_error(self):
        self.set_config()
        resp = self.app.get('/auth/oauth2callback',
                            params={'error': 'ERROR'},
                            status=302)

        expected = ('http://localhost/auth/signin?message='
                    'Google+Login+failed+%28Error+from+Google+%28ERROR%29%29')
        self.assertEqual(resp.location, expected)

    def test_callback_no_code(self):
        self.set_config()
        resp = self.app.get('/auth/oauth2callback',
                            status=302)

        expected = ('http://localhost/auth/signin?message='
                    'Google+Login+failed+%28No+authorization+code+from'
                    '+Google%29')
        self.assertEqual(resp.location, expected)

    def test_callback_missing_clientid(self):
        self.set_config({})
        resp = self.app.get('/auth/oauth2callback',
                            params={'code': 'TESTCODE'},
                            status=302)

        expected = ('http://localhost/auth/signin?message='
                    'Google+Login+failed+%28Missing+settings%29')
        self.assertEqual(resp.location, expected)

    def test_callback_missing_clientsecret(self):
        self.set_config({
            'security.google_login.client_id': 'CLIENT_ID',
        })
        resp = self.app.get('/auth/oauth2callback',
                            params={'code': 'TESTCODE'},
                            status=302)

        expected = ('http://localhost/auth/signin?message='
                    'Google+Login+failed+%28Missing+settings%29')
        self.assertEqual(resp.location, expected)

    def test_logout(self):
        self.set_config()
        self.app.set_cookie('auth_tkt', 'whatever')
        resp = self.app.get('/auth/logout',
                            status=302)

        expected = 'http://localhost/auth/signin?message=You+are+logged+out%21'
        self.assertEqual(resp.location, expected)
